
proc Set_V890_Options {} {
    global Options V890_Base

    load_Options

   if {![info exists Options(v890.channels)]} {set Options(v890.channels) 4096}
   if {![info exists Options(v890.offset)]} {set Options(v890.offset) 0}
   if {![info exists Options(v890.shift)]} {set Options(v890.shift) 1}
   if {![info exists Options(v890_GroupBase)]} {set Options(v890_GroupBase) $V890_Base}
   set V890_Base   $Options(v890_GroupBase)

   if {![info exists Options(v890.acqmode)]} {set Options(v890.acqmode) 0}
   if {![info exists Options(v890.windowwidth)]} {set Options(v890.windowwidth) 160}
   if {![info exists Options(v890.windowoffset)]} {set Options(v890.windowoffset) -150}

    update_Options
}

proc Dump_Raw_VME_Data_V890 {} {

   set data ""

   set maxmodule [inform vmecontrol inquire v890count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   for {set i 1} {$i <= $maxmodule} {incr i} {
      append data "V890#$i\n"
      execute-command 19 "Getting Raw Data for V890#$i" "1 $i" 2
      append data [read-register EXEC.Response]
      append data "\n"
   }
   return $data
}

proc define_V890_config {} {
   global VMEserver
   global V890_Base

   set maxmodule [inform vmecontrol inquire v890count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   if {$maxmodule > 0} then {DefineConfig "890 0 [format "0x%x" $V890_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v890#$i.Output_Buffer'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v767#$i.Output_Buffer"
          clear-footer
          error $m
      }
      DefineConfig "890 $i [string range [first $m] 2 end]"
   }
}

proc create_V890_spectra {} {
    global SpectrumServer V890_Base
    global Options

   set maxmodule [inform vmecontrol inquire v890count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 128

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module v890#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName v890#$i.tdc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(v890.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum v890#$i.tdc$j"
          clear-footer
          error $m
      } else {
         if {$j < 64} then {
             set group [expr $V890_Base + (($i - 1) * 2)]
             set item $j
         } else {
             set group [expr $V890_Base + (($i - 1) * 2) + 1]
             set item [expr $j - 64]
         }
          map_spectrum  "$spectrumname $group $item $Options(v890.offset) $Options(v890.shift)"
      }
    }
   }
}

proc do_V890_setup {} {
    global Options

   set maxmodule [inform vmecontrol inquire v890count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   if {$maxmodule == 0} {return}
   
   starttask v890

   for {set i 1} {$i <= $maxmodule} {incr i} {

       inform v890 click-select-module $i
               
#    acquisition mode:
#      set trigger matching
       inform v890 do_AcqMode_opcode OPT00

#    trigger options:
#      set window width 
#      set window offset 
       inform v890 do_Trig_opcode OPT10 $Options(v890.windowwidth)
       inform v890 do_Trig_opcode OPT11 $Options(v890.windowoffset)

#      set extra search margin 0
#       set reject margin 0
       inform v890 do_Trig_opcode OPT12 0
       inform v890 do_Trig_opcode OPT13 0

#     enable subtraction of trigger time
       inform v890 do_Trig_opcode OPT14 0

#    edge detection
#      set edge detection  only leading
#      set LSB 800ps
       inform v890 do_Edge_opcode OPT22 1
       inform v890 do_edge_opcode OPT24 0

#    readout 
#    disable TDC header & EOB
#    max hits 128
       inform v890 do_Readout_opcode OPT31 0
       inform v890 do_Readout_opcode OPT33 8
   }
}
